// Copyright The OpenTelemetry Authors
// SPDX-License-Identifier: Apache-2.0

// Code generated by "pdata/internal/cmd/pdatagen/main.go". DO NOT EDIT.
// To regenerate this file run "make genpdata".

package pprofile

import (
	"testing"

	"github.com/stretchr/testify/assert"

	"go.opentelemetry.io/collector/pdata/internal"
	otlpprofiles "go.opentelemetry.io/collector/pdata/internal/data/protogen/profiles/v1experimental"
)

func TestLinkSlice(t *testing.T) {
	es := NewLinkSlice()
	assert.Equal(t, 0, es.Len())
	state := internal.StateMutable
	es = newLinkSlice(&[]otlpprofiles.Link{}, &state)
	assert.Equal(t, 0, es.Len())

	emptyVal := NewLink()
	testVal := generateTestLink()
	for i := 0; i < 7; i++ {
		el := es.AppendEmpty()
		assert.Equal(t, emptyVal, es.At(i))
		fillTestLink(el)
		assert.Equal(t, testVal, es.At(i))
	}
	assert.Equal(t, 7, es.Len())
}

func TestLinkSliceReadOnly(t *testing.T) {
	sharedState := internal.StateReadOnly
	es := newLinkSlice(&[]otlpprofiles.Link{}, &sharedState)
	assert.Equal(t, 0, es.Len())
	assert.Panics(t, func() { es.AppendEmpty() })
	assert.Panics(t, func() { es.EnsureCapacity(2) })
	es2 := NewLinkSlice()
	es.CopyTo(es2)
	assert.Panics(t, func() { es2.CopyTo(es) })
	assert.Panics(t, func() { es.MoveAndAppendTo(es2) })
	assert.Panics(t, func() { es2.MoveAndAppendTo(es) })
}

func TestLinkSlice_CopyTo(t *testing.T) {
	dest := NewLinkSlice()
	// Test CopyTo to empty
	NewLinkSlice().CopyTo(dest)
	assert.Equal(t, NewLinkSlice(), dest)

	// Test CopyTo larger slice
	generateTestLinkSlice().CopyTo(dest)
	assert.Equal(t, generateTestLinkSlice(), dest)

	// Test CopyTo same size slice
	generateTestLinkSlice().CopyTo(dest)
	assert.Equal(t, generateTestLinkSlice(), dest)
}

func TestLinkSlice_EnsureCapacity(t *testing.T) {
	es := generateTestLinkSlice()

	// Test ensure smaller capacity.
	const ensureSmallLen = 4
	es.EnsureCapacity(ensureSmallLen)
	assert.Less(t, ensureSmallLen, es.Len())
	assert.Equal(t, es.Len(), cap(*es.orig))
	assert.Equal(t, generateTestLinkSlice(), es)

	// Test ensure larger capacity
	const ensureLargeLen = 9
	es.EnsureCapacity(ensureLargeLen)
	assert.Less(t, generateTestLinkSlice().Len(), ensureLargeLen)
	assert.Equal(t, ensureLargeLen, cap(*es.orig))
	assert.Equal(t, generateTestLinkSlice(), es)
}

func TestLinkSlice_MoveAndAppendTo(t *testing.T) {
	// Test MoveAndAppendTo to empty
	expectedSlice := generateTestLinkSlice()
	dest := NewLinkSlice()
	src := generateTestLinkSlice()
	src.MoveAndAppendTo(dest)
	assert.Equal(t, generateTestLinkSlice(), dest)
	assert.Equal(t, 0, src.Len())
	assert.Equal(t, expectedSlice.Len(), dest.Len())

	// Test MoveAndAppendTo empty slice
	src.MoveAndAppendTo(dest)
	assert.Equal(t, generateTestLinkSlice(), dest)
	assert.Equal(t, 0, src.Len())
	assert.Equal(t, expectedSlice.Len(), dest.Len())

	// Test MoveAndAppendTo not empty slice
	generateTestLinkSlice().MoveAndAppendTo(dest)
	assert.Equal(t, 2*expectedSlice.Len(), dest.Len())
	for i := 0; i < expectedSlice.Len(); i++ {
		assert.Equal(t, expectedSlice.At(i), dest.At(i))
		assert.Equal(t, expectedSlice.At(i), dest.At(i+expectedSlice.Len()))
	}
}

func TestLinkSlice_RemoveIf(t *testing.T) {
	// Test RemoveIf on empty slice
	emptySlice := NewLinkSlice()
	emptySlice.RemoveIf(func(el Link) bool {
		t.Fail()
		return false
	})

	// Test RemoveIf
	filtered := generateTestLinkSlice()
	pos := 0
	filtered.RemoveIf(func(el Link) bool {
		pos++
		return pos%3 == 0
	})
	assert.Equal(t, 5, filtered.Len())
}

func generateTestLinkSlice() LinkSlice {
	es := NewLinkSlice()
	fillTestLinkSlice(es)
	return es
}

func fillTestLinkSlice(es LinkSlice) {
	*es.orig = make([]otlpprofiles.Link, 7)
	for i := 0; i < 7; i++ {
		(*es.orig)[i] = otlpprofiles.Link{}
		fillTestLink(newLink(&(*es.orig)[i], es.state))
	}
}
