// Copyright The OpenTelemetry Authors
// SPDX-License-Identifier: Apache-2.0

// Code generated by "pdata/internal/cmd/pdatagen/main.go". DO NOT EDIT.
// To regenerate this file run "make genpdata".

package pprofile

import (
	"testing"

	"github.com/stretchr/testify/assert"

	"go.opentelemetry.io/collector/pdata/internal"
	otlpprofiles "go.opentelemetry.io/collector/pdata/internal/data/protogen/profiles/v1experimental"
)

func TestSampleSlice(t *testing.T) {
	es := NewSampleSlice()
	assert.Equal(t, 0, es.Len())
	state := internal.StateMutable
	es = newSampleSlice(&[]otlpprofiles.Sample{}, &state)
	assert.Equal(t, 0, es.Len())

	emptyVal := NewSample()
	testVal := generateTestSample()
	for i := 0; i < 7; i++ {
		el := es.AppendEmpty()
		assert.Equal(t, emptyVal, es.At(i))
		fillTestSample(el)
		assert.Equal(t, testVal, es.At(i))
	}
	assert.Equal(t, 7, es.Len())
}

func TestSampleSliceReadOnly(t *testing.T) {
	sharedState := internal.StateReadOnly
	es := newSampleSlice(&[]otlpprofiles.Sample{}, &sharedState)
	assert.Equal(t, 0, es.Len())
	assert.Panics(t, func() { es.AppendEmpty() })
	assert.Panics(t, func() { es.EnsureCapacity(2) })
	es2 := NewSampleSlice()
	es.CopyTo(es2)
	assert.Panics(t, func() { es2.CopyTo(es) })
	assert.Panics(t, func() { es.MoveAndAppendTo(es2) })
	assert.Panics(t, func() { es2.MoveAndAppendTo(es) })
}

func TestSampleSlice_CopyTo(t *testing.T) {
	dest := NewSampleSlice()
	// Test CopyTo to empty
	NewSampleSlice().CopyTo(dest)
	assert.Equal(t, NewSampleSlice(), dest)

	// Test CopyTo larger slice
	generateTestSampleSlice().CopyTo(dest)
	assert.Equal(t, generateTestSampleSlice(), dest)

	// Test CopyTo same size slice
	generateTestSampleSlice().CopyTo(dest)
	assert.Equal(t, generateTestSampleSlice(), dest)
}

func TestSampleSlice_EnsureCapacity(t *testing.T) {
	es := generateTestSampleSlice()

	// Test ensure smaller capacity.
	const ensureSmallLen = 4
	es.EnsureCapacity(ensureSmallLen)
	assert.Less(t, ensureSmallLen, es.Len())
	assert.Equal(t, es.Len(), cap(*es.orig))
	assert.Equal(t, generateTestSampleSlice(), es)

	// Test ensure larger capacity
	const ensureLargeLen = 9
	es.EnsureCapacity(ensureLargeLen)
	assert.Less(t, generateTestSampleSlice().Len(), ensureLargeLen)
	assert.Equal(t, ensureLargeLen, cap(*es.orig))
	assert.Equal(t, generateTestSampleSlice(), es)
}

func TestSampleSlice_MoveAndAppendTo(t *testing.T) {
	// Test MoveAndAppendTo to empty
	expectedSlice := generateTestSampleSlice()
	dest := NewSampleSlice()
	src := generateTestSampleSlice()
	src.MoveAndAppendTo(dest)
	assert.Equal(t, generateTestSampleSlice(), dest)
	assert.Equal(t, 0, src.Len())
	assert.Equal(t, expectedSlice.Len(), dest.Len())

	// Test MoveAndAppendTo empty slice
	src.MoveAndAppendTo(dest)
	assert.Equal(t, generateTestSampleSlice(), dest)
	assert.Equal(t, 0, src.Len())
	assert.Equal(t, expectedSlice.Len(), dest.Len())

	// Test MoveAndAppendTo not empty slice
	generateTestSampleSlice().MoveAndAppendTo(dest)
	assert.Equal(t, 2*expectedSlice.Len(), dest.Len())
	for i := 0; i < expectedSlice.Len(); i++ {
		assert.Equal(t, expectedSlice.At(i), dest.At(i))
		assert.Equal(t, expectedSlice.At(i), dest.At(i+expectedSlice.Len()))
	}
}

func TestSampleSlice_RemoveIf(t *testing.T) {
	// Test RemoveIf on empty slice
	emptySlice := NewSampleSlice()
	emptySlice.RemoveIf(func(el Sample) bool {
		t.Fail()
		return false
	})

	// Test RemoveIf
	filtered := generateTestSampleSlice()
	pos := 0
	filtered.RemoveIf(func(el Sample) bool {
		pos++
		return pos%3 == 0
	})
	assert.Equal(t, 5, filtered.Len())
}

func generateTestSampleSlice() SampleSlice {
	es := NewSampleSlice()
	fillTestSampleSlice(es)
	return es
}

func fillTestSampleSlice(es SampleSlice) {
	*es.orig = make([]otlpprofiles.Sample, 7)
	for i := 0; i < 7; i++ {
		(*es.orig)[i] = otlpprofiles.Sample{}
		fillTestSample(newSample(&(*es.orig)[i], es.state))
	}
}
